# coding=utf-8
import logging

from django.contrib.auth.decorators import login_required
from django.shortcuts import redirect, get_object_or_404

from web.forms import WorkerForm
from web.forms import AvailabilityAddForm, AvailabilityEditForm, HolidayAddForm
from web.models.worker_study_role import STUDY_ROLE_CHOICES, HEALTH_PARTNER_ROLE_CHOICES, VOUCHER_PARTNER_ROLE_CHOICES, \
    WORKER_STAFF, WORKER_HEALTH_PARTNER, WORKER_VOUCHER_PARTNER
from . import wrap_response
from ..models import Worker, Availability, Holiday
from ..models.constants import WEEKDAY_CHOICES, GLOBAL_STUDY_ID

logger = logging.getLogger(__name__)


def worker_list(request, worker_type=WORKER_STAFF):
    role_choices = role_choices_by_worker_type(worker_type)
    roles = []

    for role_type, role_name in role_choices:
        roles.append(role_type)

    doctors_list = Worker.objects.filter(roles__study_id=GLOBAL_STUDY_ID, roles__role__in=roles).order_by(
        '-last_name')
    context = {
        'doctors_list': doctors_list,
        'worker_type': worker_type
    }

    return wrap_response(request, "doctors/index.html", context)


def role_choices_by_worker_type(worker_type):
    if worker_type == WORKER_STAFF:
        return STUDY_ROLE_CHOICES
    elif worker_type == WORKER_HEALTH_PARTNER:
        return HEALTH_PARTNER_ROLE_CHOICES
    elif worker_type == WORKER_VOUCHER_PARTNER:
        return VOUCHER_PARTNER_ROLE_CHOICES
    else:
        return ()


def worker_add(request, worker_type):
    if request.method == 'POST':
        form = WorkerForm(request.POST, request.FILES, role_choices=role_choices_by_worker_type(worker_type))
        if form.is_valid():
            form.save()
            return redirect('web.views.workers')
    else:
        form = WorkerForm(role_choices=role_choices_by_worker_type(worker_type))

    return wrap_response(request, 'doctors/add.html', {'form': form})


def role_choices_by_worker(worker):
    roles = worker.roles.filter(study=GLOBAL_STUDY_ID)
    if roles.count() == 0:
        return STUDY_ROLE_CHOICES
    role = roles[0]
    for role_type, role_name in STUDY_ROLE_CHOICES.items():
        if role_type == role:
            return STUDY_ROLE_CHOICES
    for role_type, role_name in HEALTH_PARTNER_ROLE_CHOICES.items():
        if role_type == role:
            return HEALTH_PARTNER_ROLE_CHOICES
    for role_type, role_name in VOUCHER_PARTNER_ROLE_CHOICES.items():
        if role_type == role:
            return VOUCHER_PARTNER_ROLE_CHOICES
    return STUDY_ROLE_CHOICES


def worker_edit(request, worker_id):
    worker = get_object_or_404(Worker, id=worker_id)
    if request.method == 'POST':
        form = WorkerForm(request.POST, request.FILES, instance=worker, role_choices=role_choices_by_worker(worker))
        if form.is_valid():
            form.save()
            return redirect('web.views.workers')
    else:
        form = WorkerForm(instance=worker)
    availabilities = Availability.objects.filter(person=worker_id).order_by('day_number', 'available_from')
    holidays = Holiday.objects.filter(person=worker_id).order_by('-datetime_start')
    return wrap_response(request, 'doctors/edit.html',
                         {
                             'form': form,
                             'availabilities': availabilities,
                             'holidays': holidays,
                             'doctor_id': worker_id,
                             'weekdays': WEEKDAY_CHOICES
                         })


def doctor_disable(request, doctor_id):
    the_doctor = get_object_or_404(Worker, id=doctor_id)
    the_doctor.disable()
    return worker_list(request)


@login_required
def worker_availability_delete(request, availability_id):
    availability = Availability.objects.filter(id=availability_id)
    doctor_id = availability[0].person.id
    availability.delete()
    return redirect(worker_edit, doctor_id=doctor_id)


def worker_availability_add(request, doctor_id):
    worker = get_object_or_404(Worker, id=doctor_id)
    if request.method == 'POST':
        form = AvailabilityAddForm(request.POST, request.FILES)
        if form.is_valid():
            form.save()
            return redirect(worker_edit, doctor_id=doctor_id)
    else:
        form = AvailabilityAddForm(initial={'person': worker})
    return wrap_response(request, 'doctors/add_availability.html',
                         {
                             'form': form,
                             'doctor_id': doctor_id
                         })


def worker_availability_edit(request, availability_id):
    availability = get_object_or_404(Availability, id=availability_id)
    if request.method == 'POST':
        form = AvailabilityEditForm(request.POST, request.FILES, instance=availability)
        if form.is_valid():
            form.save()
            return redirect(worker_edit, doctor_id=availability.person_id)
    else:
        form = AvailabilityEditForm(instance=availability)
    return wrap_response(request, 'doctors/edit_availability.html',
                         {
                             'form': form,
                             'availability_id': availability_id,
                             'doctor_id': availability.person_id,
                         })


@login_required
def worker_holiday_delete(request, holiday_id):
    holiday = Holiday.objects.filter(id=holiday_id)
    doctor_id = holiday[0].person.id
    holiday.delete()
    return redirect(worker_edit, doctor_id=doctor_id)


def worker_holiday_add(request, doctor_id):
    doctors = Worker.objects.filter(id=doctor_id)
    doctor = None
    if len(doctors) > 0:
        doctor = doctors[0]
    if request.method == 'POST':
        form = HolidayAddForm(request.POST, request.FILES)
        if form.is_valid():
            form.save()
            return redirect(worker_edit, doctor_id=doctor_id)
    else:
        form = HolidayAddForm(initial={'person': doctor})
    return wrap_response(request, 'doctors/add_holiday.html',
                         {
                             'form': form,
                             'doctor_id': doctor_id
                         })
